package sun.security.provider;

import java.util.Arrays;

public final class SHA extends DigestBase
{
    private int[] W;
    private int[] state;
    private static final int round1_kt = 1518500249;
    private static final int round2_kt = 1859775393;
    private static final int round3_kt = -1894007588;
    private static final int round4_kt = -899497514;
    
    public SHA() {
        super("SHA-1", 20, 64);
        this.state = new int[5];
        this.W = new int[80];
        this.resetHashes();
    }
    
    @Override
    public Object clone() throws CloneNotSupportedException {
        SHA sha = (SHA)super.clone();
        sha.state = (int[])sha.state.clone();
        sha.W = new int[80];
        return sha;
    }
    
    @Override
    void implReset() {
        this.resetHashes();
        Arrays.fill(this.W, 0);
    }
    
    private void resetHashes() {
        this.state[0] = 1732584193;
        this.state[1] = -271733879;
        this.state[2] = -1732584194;
        this.state[3] = 271733878;
        this.state[4] = -1009589776;
    }
    
    @Override
    void implDigest(byte[] array, int n) {
        long n2 = this.bytesProcessed << 3;
        int n3 = (int)this.bytesProcessed & 0x3F;
        this.engineUpdate(SHA.padding, 0, (n3 < 56) ? (56 - n3) : (120 - n3));
        ByteArrayAccess.i2bBig4((int)(n2 >>> 32), this.buffer, 56);
        ByteArrayAccess.i2bBig4((int)n2, this.buffer, 60);
        this.implCompress(this.buffer, 0);
        ByteArrayAccess.i2bBig(this.state, 0, array, n, 20);
    }
    
    @Override
    void implCompress(byte[] array, int n) {
        ByteArrayAccess.b2iBig64(array, n, this.W);
        for (int i = 16; i <= 79; ++i) {
            int n2 = this.W[i - 3] ^ this.W[i - 8] ^ this.W[i - 14] ^ this.W[i - 16];
            this.W[i] = (n2 << 1 | n2 >>> 31);
        }
        int n3 = this.state[0];
        n3 = this.state[1];
        int n4 = this.state[2];
        n4 = this.state[3];
        n4 = this.state[4];
        for (int j = 0; j < 20; ++j) {
            int n5 = (n3 << 5 | n3 >>> 27) + ((n3 & n4) | (~n3 & n4)) + n4 + this.W[j] + 1518500249;
            n4 = (n4 = n4);
            n4 = (n3 << 30 | n3 >>> 2);
            n3 = n5;
        }
        for (int k = 20; k < 40; ++k) {
            int n6 = (n3 << 5 | n3 >>> 27) + (n3 ^ n4 ^ n4) + n4 + this.W[k] + 1859775393;
            n4 = (n4 = n4);
            n4 = (n3 << 30 | n3 >>> 2);
            n3 = n6;
        }
        for (int l = 40; l < 60; ++l) {
            int n7 = (n3 << 5 | n3 >>> 27) + ((n3 & n4) | (n3 & n4) | (n4 & n4)) + n4 + this.W[l] - 1894007588;
            n4 = (n4 = n4);
            n4 = (n3 << 30 | n3 >>> 2);
            n3 = n7;
        }
        for (int n8 = 60; n8 < 80; ++n8) {
            int n9 = (n3 << 5 | n3 >>> 27) + (n3 ^ n4 ^ n4) + n4 + this.W[n8] - 899497514;
            n4 = (n4 = n4);
            n4 = (n3 << 30 | n3 >>> 2);
            n3 = n9;
        }
        int[] state = this.state;
        int n10 = 0;
        state[n10] += n3;
        int[] state2 = this.state;
        int n11 = 1;
        state2[n11] += n3;
        int[] state3 = this.state;
        int n12 = 2;
        state3[n12] += n4;
        int[] state4 = this.state;
        int n13 = 3;
        state4[n13] += n4;
        int[] state5 = this.state;
        int n14 = 4;
        state5[n14] += n4;
    }
}
